@@ -23,7 +23,7 @@ from collections import defaultdict
 
 import collections
 import heapq
-import itertools
+from itertools import chain
 
 from dulwich.diff_tree import (
     RENAME_CHANGE_TYPES,
@@ -100,7 +100,7 @@ class _CommitTimeQueue(object):
         self._extra_commits_left = _MAX_EXTRA_COMMITS
         self._is_finished = False
 
-        for commit_id in itertools.chain(walker.include, walker.excluded):
+        for commit_id in chain(walker.include, walker.excluded):
             self._push(commit_id)
 
     def _push(self, commit_id):
@@ -181,6 +181,8 @@ class _CommitTimeQueue(object):
         self._is_finished = True
         return None
 
+    __next__ = next
+
 
 class Walker(object):
     """Object for performing a walk of commits in a store.
@@ -303,7 +305,7 @@ class Walker(object):
     def _next(self):
         max_entries = self.max_entries
         while max_entries is None or self._num_entries < max_entries:
-            entry = self._queue.next()
+            entry = next(self._queue)
             if entry is not None:
                 self._out_queue.append(entry)
             if entry is None or len(self._out_queue) > _MAX_EXTRA_COMMITS:
